Package: survMisc
Type: Package
Version: 0.5.6
Date: 2022-04-07
Depends: survival
Imports: graphics, grDevices, stats, utils, knitr, KMsurv, ggplot2,
        data.table, zoo, grid, gridExtra, km.ci, xtable
Author: Chris Dardis
Maintainer: Chris Dardis <christopherdardis@gmail.com>
License: GPL-2
Title: Miscellaneous Functions for Survival Data
Description: A collection of functions to help in the analysis of
    right-censored survival data. These extend the methods available in
    package:survival.
BugReports: https://github.com/dardisco/survMisc/issues
LazyData: true
VignetteBuilder: knitr
Collate: 'ten.R' 'nc.R' 'sf.R' 'ci.R' 'autoplotTAP.R' 'autoplotTen.R'
        'print.R' 'asWide.R' 'COV.R' 'predict.R' 'comp.R' 'cutp.R'
        'gastric.R' 'gof.R' 'onAttach.R' 'profLik.R' 'rsq.R'
        'survMisc_package.R' 'xtable.R'
NeedsCompilation: no
RoxygenNote: 7.1.2
Packaged: 2022-04-07 15:42:53 UTC; cd
Repository: CRAN
Date/Publication: 2022-04-07 16:10:02 UTC
